Package: bio3d
Title: Biological Structure Analysis
Version: 2.4-5
Authors@R: c(person("Barry", "Grant", role=c("aut", "cre"),
                     email="bjgrant@ucsd.edu"),
             person("Xin-Qiu", "Yao", role="aut"),
             person("Lars", "Skjaerven", role="aut"), 
             person("Julien", "Ide", role="aut"))
Author: Barry Grant [aut, cre],
  Xin-Qiu Yao [aut],
  Lars Skjaerven [aut],
  Julien Ide [aut]
VignetteBuilder: knitr
LinkingTo: Rcpp
Imports: Rcpp, parallel, grid, graphics, grDevices, stats, utils
Suggests: XML, RCurl, lattice, ncdf4, igraph, bigmemory, knitr,
        rmarkdown, testthat (>= 0.9.1), httr, msa, Biostrings
Depends: R (>= 3.1.0)
LazyData: yes
Description: Utilities to process, organize and explore protein structure,
    sequence and dynamics data. Features include the ability to read and write
    structure, sequence and dynamic trajectory data, perform sequence and structure
    database searches, data summaries, atom selection, alignment, superposition,
    rigid core identification, clustering, torsion analysis, distance matrix
    analysis, structure and sequence conservation analysis, normal mode analysis,
    principal component analysis of heterogeneous structure data, and correlation
    network analysis from normal mode and molecular dynamics data. In addition,
    various utility functions are provided to enable the statistical and graphical
    power of the R environment to work with biological sequence and structural data.
    Please refer to the URLs below for more information.
Maintainer: Barry Grant <bjgrant@ucsd.edu>
License: GPL (>= 2)
URL: http://thegrantlab.org/bio3d/,
        https://bitbucket.org/Grantlab/bio3d/
RoxygenNote: 7.1.1
NeedsCompilation: yes
Packaged: 2024-10-29 13:53:45 UTC; xyao
Repository: CRAN
Date/Publication: 2024-10-29 23:40:10 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 10:32:45 UTC; unix
Archs: bio3d.so.dSYM
