#exportPattern("^[[:alpha:]]+")
exportPattern("^[^\\.]")
useDynLib(bio3d)
import("graphics")
import("grDevices")
import("stats")
import("utils")
import(parallel, grid, Rcpp)
S3method(as.pdb, default)
S3method(as.pdb, mol2)
S3method(as.pdb, prmtop)
S3method(atom2ele, default)
S3method(atom2ele, pdb)
S3method(atom2mass, default)
S3method(atom2mass, pdb)
S3method(atom.select, pdb)
S3method(atom.select, mol2)
S3method(atom.select, prmtop)
S3method(atom.select, pdbs)
S3method(bhattacharyya, array)
S3method(bhattacharyya, enma)
S3method(bhattacharyya, matrix)
S3method(bhattacharyya, nma)
S3method(bhattacharyya, pca)
S3method(cmap, default)
S3method(cmap, pdb)
S3method(cmap, pdbs)
S3method(cmap, xyz)
S3method(cna, dccm)
S3method(cna, ensmb)
S3method(com, pdb)
S3method(com, xyz)
S3method(core.find, default)
S3method(core.find, pdb)
S3method(core.find, pdbs)
S3method(covsoverlap, enma)
S3method(covsoverlap, nma)
S3method(dccm, enma)
S3method(dccm, nma)
S3method(dccm, pca)
S3method(dccm, xyz)
S3method(dccm, gnm)
S3method(dccm, egnm)
S3method(dm, pdb)
S3method(dm, xyz)
S3method(dm, pdbs)
S3method(dssp, pdb)
S3method(dssp, pdbs)
S3method(dssp, xyz)
S3method(geostas, default)
S3method(geostas, enma)
S3method(geostas, nma)
S3method(geostas, pdb)
S3method(geostas, pdbs)
S3method(geostas, xyz)
S3method(pdbfit, pdb)
S3method(pdbfit, pdbs)
S3method(identify, cna)
S3method(mktrj, enma)
S3method(mktrj, nma)
S3method(mktrj, pca)
S3method(nma, pdb)
S3method(nma, pdbs)
S3method(aanma, pdb)
S3method(aanma, pdbs)
S3method(gnm, pdb)
S3method(gnm, pdbs)
S3method(pca, array)
S3method(pca, pdbs)
S3method(pca, tor)
S3method(pca, xyz)
S3method(plot, bio3d)
S3method(plot, blast)
S3method(plot, cmap)
S3method(plot, cna)
S3method(plot, ecna)
S3method(plot, cnapath)
S3method(plot, ecnapath)
S3method(plot, core)
S3method(plot, dccm)
S3method(plot, dmat)
S3method(plot, enma)
S3method(plot, fasta)
S3method(plot, fluct)
S3method(plot, geostas)
S3method(plot, hmmer)
S3method(plot, nma)
S3method(plot, pca)
S3method(plot, pca.loadings)
S3method(plot, pca.score)
S3method(plot, pca.scree)
S3method(plot, rmsip)
S3method(plot, matrix.loadings)
S3method(print, core)
S3method(print, enma)
S3method(print, fasta)
S3method(print, nma)
S3method(print, pca)
S3method(print, pdb)
S3method(print, rle2)
S3method(print, select)
S3method(print, sse)
S3method(print, xyz)
S3method(print, cna)
S3method(print, cnapath)
S3method(print, geostas)
S3method(print, mol2)
S3method(print, prmtop)
S3method(read.crd, amber)
S3method(read.crd, charmm)
S3method(rmsip, default)
S3method(rmsip, enma)
S3method(sip, default)
S3method(sip, enma)
S3method(sip, nma)
S3method(summary, pdb)
S3method(summary, cna)
S3method(summary, cnapath)
S3method(trim, pdb)
S3method(trim, mol2)
S3method(trim, pdbs)
S3method(trim, xyz)
S3method(pymol, dccm)
S3method(pymol, modes)
S3method(pymol, nma)
S3method(pymol, pca)
S3method(pymol, pdbs)
S3method(mask, dccm)
S3method(vmd, cna)
S3method(vmd, ecna)
S3method(vmd, cnapath)
S3method(vmd, ecnapath)

