# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

.read_cif <- function(filename, maxlines = -1L, multi = FALSE) {
    .Call('_bio3d_read_cif', PACKAGE = 'bio3d', filename, maxlines, multi)
}

.read_crd <- function(filename) {
    .Call('_bio3d_read_crd', PACKAGE = 'bio3d', filename)
}

.read_pdb <- function(filename, multi = FALSE, hex = FALSE, maxlines = -1L, atoms_only = FALSE) {
    .Call('_bio3d_read_pdb', PACKAGE = 'bio3d', filename, multi, hex, maxlines, atoms_only)
}

.read_prmtop <- function(filename) {
    .Call('_bio3d_read_prmtop', PACKAGE = 'bio3d', filename)
}

